"use strict";
var _errors_1 = require("./_errors");
var create_promise_1 = require("../promise/create-promise");
var _results_1 = require("./_results");
require("reflect-metadata");
var TestRunner = (function () {
    function TestRunner() {
        var _this = this;
        this._currentTestId = 0;
        this._currentTestFixtureIndex = 0;
        this._currentTestIndex = 0;
        this._currentTestCaseIndex = 0;
        this._resultPromise = create_promise_1.createPromise();
        this._testResults = new _results_1.TestSetResults();
        this._getTestDescription = function (test, testCaseArguments) {
            var testDescription = _this._currentTestId + " " + (test.ignored ? "# skip " : "") + test.description;
            if (testCaseArguments !== undefined) {
                testDescription += " [ " + testCaseArguments.map(function (x) { return JSON.stringify(x) || "undefined"; }).join(", ") + " ]";
            }
            return testDescription;
        };
        this._runNextTestCase = function () {
            _this._currentTestCaseIndex++;
            if (!_this._testFixtures[_this._currentTestFixtureIndex].tests[_this._currentTestIndex].testCases[_this._currentTestCaseIndex]) {
                _this._runNextTest();
            }
            else {
                _this._runTest(_this._testFixtures[_this._currentTestFixtureIndex], _this._testFixtures[_this._currentTestFixtureIndex].tests[_this._currentTestIndex], _this._testFixtures[_this._currentTestFixtureIndex].tests[_this._currentTestIndex].testCases[_this._currentTestCaseIndex].arguments);
            }
        };
    }
    TestRunner.prototype.run = function (testSet) {
        var _this = this;
        var anyTestsFocussed = testSet.testFixtures.filter(function (testFixture) { return testFixture.focussed || testFixture.tests.filter(function (test) { return test.focussed; }).length > 0; }).length > 0;
        this._testFixtures = testSet.testFixtures;
        // Filter out unfocussed tests if any are focussed
        if (anyTestsFocussed) {
            this._testFixtures = testSet.testFixtures.filter(function (testFixture) { return testFixture.focussed || testFixture.tests.filter(function (test) { return test.focussed; }).length > 0; });
        }
        var totalTestCount = this._testFixtures
            .filter(function (x) { return x.tests.length > 0; })
            .map(function (x) { return x.tests.map(function (y) { return y.testCases.length; })
            .reduce(function (a, b) { return a + b; }, 0); })
            .reduce(function (a, b) { return a + b; }, 0);
        if (totalTestCount === 0) {
            throw new Error("no tests to run.");
        }
        else {
            process.stdout.write("TAP version 13\n");
            process.stdout.write("1.." + totalTestCount + "\n");
            this._currentTestFixtureResults = this._testResults.addTestFixtureResult(this._testFixtures[this._currentTestFixtureIndex]);
            this._currentTestResults = this._currentTestFixtureResults.addTestResult(this._testFixtures[this._currentTestFixtureIndex].tests[this._currentTestIndex]);
            setTimeout(function () {
                _this._runTest(_this._testFixtures[_this._currentTestFixtureIndex], _this._testFixtures[_this._currentTestFixtureIndex].tests[_this._currentTestIndex], _this._testFixtures[_this._currentTestFixtureIndex].tests[_this._currentTestIndex].testCases[_this._currentTestCaseIndex].arguments);
            });
        }
        return this._resultPromise;
    };
    TestRunner.prototype._runTest = function (testFixture, test, testCaseArguments) {
        var _this = this;
        this._currentTestId++;
        if (test.ignored) {
            process.stdout.write("ok " + this._getTestDescription(test, testCaseArguments) + "\n");
            this._runNextTest();
            return;
        }
        var setupFunctions = Reflect.getMetadata("alsatian:setup", testFixture.fixture);
        if (setupFunctions) {
            setupFunctions.forEach(function (setupFunction) {
                testFixture.fixture[setupFunction].call(testFixture.fixture);
            });
        }
        try {
            if (test.isAsync) {
                var timeout_1 = false;
                var promise = testFixture.fixture[test.key].apply(testFixture.fixture, testCaseArguments);
                var timeoutCheck_1 = null;
                promise.then(function () {
                    if (!timeout_1) {
                        _this._notifySuccess(test, testCaseArguments);
                        clearTimeout(timeoutCheck_1);
                    }
                })
                    .catch(function (error) {
                    _this._handleError(error, test, testCaseArguments);
                });
                timeoutCheck_1 = setTimeout(function () {
                    timeout_1 = true;
                    _this._handleError(new Error("The test exceeded the given timeout."), test, testCaseArguments);
                }, test.timeout | 500);
            }
            else {
                testFixture.fixture[test.key].apply(testFixture, testCaseArguments);
                this._notifySuccess(test, testCaseArguments);
            }
        }
        catch (error) {
            this._handleError(error, test, testCaseArguments);
        }
    };
    TestRunner.prototype._handleError = function (error, test, testCaseArguments) {
        this._teardown();
        this._currentTestResults.addTestCaseResult(testCaseArguments, error);
        process.stdout.write("not ok " + this._getTestDescription(test, testCaseArguments) + "\n");
        if (error instanceof _errors_1.MatchError) {
            process.stdout.write(" ---\n   message: \"" + error.message + "\"\n   severity: fail\n   data:\n     got: " + JSON.stringify(error.actualValue) + "\n     expect: " + JSON.stringify(error.expectedValue) + "\n ...\n");
        }
        else {
            console.log(error);
            process.stdout.write("# Unknown Error\n");
        }
        this._runNextTestCase();
    };
    TestRunner.prototype._notifySuccess = function (test, testCaseArguments) {
        this._teardown();
        process.stdout.write("ok " + this._getTestDescription(test, testCaseArguments) + "\n");
        this._currentTestResults.addTestCaseResult(testCaseArguments);
        this._runNextTestCase();
    };
    TestRunner.prototype._teardown = function () {
        var testFixture = this._testFixtures[this._currentTestFixtureIndex];
        var teardownFunctions = Reflect.getMetadata("alsatian:teardown", testFixture.fixture);
        if (teardownFunctions) {
            teardownFunctions.forEach(function (teardownFunction) {
                testFixture.fixture[teardownFunction].call(testFixture.fixture);
            });
        }
    };
    TestRunner.prototype._exit = function () {
        this._resultPromise.resolve(this._testResults);
    };
    TestRunner.prototype._runNextTestFixture = function () {
        var _this = this;
        this._currentTestFixtureIndex++;
        this._currentTestIndex = 0;
        this._currentTestCaseIndex = 0;
        // no more test fixtures
        if (!this._testFixtures[this._currentTestFixtureIndex]) {
            this._exit();
        }
        else if (!this._testFixtures[this._currentTestFixtureIndex].tests[this._currentTestIndex]) {
            this._runNextTestFixture();
        }
        else {
            this._currentTestFixtureResults = this._testResults.addTestFixtureResult(this._testFixtures[this._currentTestFixtureIndex]);
            setTimeout(function () {
                _this._runTest(_this._testFixtures[_this._currentTestFixtureIndex], _this._testFixtures[_this._currentTestFixtureIndex].tests[_this._currentTestIndex], _this._testFixtures[_this._currentTestFixtureIndex].tests[_this._currentTestIndex].testCases[_this._currentTestCaseIndex].arguments);
            });
        }
    };
    TestRunner.prototype._runNextTest = function () {
        this._currentTestIndex++;
        this._currentTestCaseIndex = 0;
        if (!this._testFixtures[this._currentTestFixtureIndex].tests[this._currentTestIndex]) {
            this._runNextTestFixture();
        }
        else {
            this._currentTestResults = this._currentTestFixtureResults.addTestResult(this._testFixtures[this._currentTestFixtureIndex].tests[this._currentTestIndex]);
            this._runTest(this._testFixtures[this._currentTestFixtureIndex], this._testFixtures[this._currentTestFixtureIndex].tests[this._currentTestIndex], this._testFixtures[this._currentTestFixtureIndex].tests[this._currentTestIndex].testCases[this._currentTestCaseIndex].arguments);
        }
    };
    return TestRunner;
}());
exports.TestRunner = TestRunner;
//# sourceMappingURL=test-runner.js.map